#include <windows.h>
#include <cassert>
#include <cmath>

static char g_szClassName[] = "MyWindowClass";
static HINSTANCE g_hInst = NULL;

const UINT idTimer1 = 1;
UINT nTimerDelay = 10; 


struct Kulka
{
  Kulka (HWND hwnd, double xx, double yy, double vvx, double vvy, 
         COLORREF kolor_kulki, int rozmiar = 20)
  : x(xx), y(yy), vx(vvx), vy(vvy), 
    _kolor_kulki(kolor_kulki), _rozmiar(rozmiar), 
    _pedzel_kulki(::CreateSolidBrush(kolor_kulki)),
    _hwnd(hwnd)
  {
      Kulka::_max_screen_x = GetSystemMetrics(SM_CXSCREEN) - _rozmiar;
      Kulka::_max_screen_y = GetSystemMetrics(SM_CYSCREEN) - _rozmiar;
  }
  
  ~Kulka() 
  {
    ::DeleteObject(_pedzel_kulki);
  }
  double x;
  double y;
  double vx;
  double vy;
  
  void Odrysuj(HDC hdc);
  void Usun(HDC hdc, COLORREF kolor);
  void Przesun(double dt);
  void SprawdzZderzenia(RECT const& rc, RECT const& v);

  POINT pixel(double x, double y);

private:  
  static int _max_screen_x;
  static int _max_screen_y;

   
  COLORREF _kolor_kulki;
  int _rozmiar;
  HBRUSH _pedzel_kulki;

  HWND _hwnd;
};

int Kulka::_max_screen_x = GetSystemMetrics(SM_CXSCREEN);
int Kulka::_max_screen_y = GetSystemMetrics(SM_CYSCREEN);



void Kulka::Usun(HDC hdc, COLORREF kolor)
{
   POINT punkt = pixel(x, y);
   RECT rc;
   rc.left = punkt.x;
   rc.top =  punkt.y;
   rc.right = rc.left + _rozmiar;
   rc.bottom = rc.top + _rozmiar;
   FillRect(hdc, &rc, (HBRUSH)(kolor + 1));  
}

void Kulka::Odrysuj(HDC hdc)
{

   POINT punkt = pixel(x,y);
   int xx = punkt.x;
   int yy = punkt.y;
   
   HBRUSH old_brush = (HBRUSH)::SelectObject(hdc, _pedzel_kulki);
   ::Ellipse(hdc, xx, yy,  xx + _rozmiar, yy + _rozmiar);
   ::SelectObject(hdc, old_brush);
   GdiFlush();
}


void Kulka::Przesun(double dt)
{
   x += vx*dt;
   y += vy*dt;
   vx *= 0.999;
   vy *= 0.999;
} 

void Kulka::SprawdzZderzenia(RECT const& client, RECT const& v)
{
   const double spowalniacz = 5.0;

   POINT p0 = {client.left, client.top};
   POINT p1 = {client.right - _rozmiar, client.bottom - _rozmiar};
   ::ClientToScreen(_hwnd, &p0);
   ::ClientToScreen(_hwnd, &p1);
   double lewy  = p0.x/double(_max_screen_x);
   double prawy = p1.x/double(_max_screen_x);
   double gorny = p0.y/double(_max_screen_y);
   double dolny = p1.y/double(_max_screen_y);

   if(x < lewy)  { x = lewy;  vx = -vx + v.left/spowalniacz; }
   if(x > prawy) { x = prawy; vx = -vx + v.right/spowalniacz; }
   if(y > dolny) { y = dolny; vy = -vy + v.bottom/spowalniacz; }
   if(y < gorny) { y = gorny; vy = -vy + v.top/spowalniacz ; }

}



POINT Kulka::pixel(double x, double y)
{
 //  assert (x >= 0 && x <= 1.0);
 //  assert (y >= 0 && y <= 1.0);

   POINT wynik;
   wynik.x = static_cast<int> (x * _max_screen_x);
   wynik.y = static_cast<int> (y * _max_screen_y);
   ScreenToClient(_hwnd, &wynik);
   return wynik;
}






LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
  const double dt = 0.003;
  static Kulka* p_kulka = 0;
  
  switch(Message)
   {
      case WM_CREATE:
         SetTimer(hwnd, idTimer1, nTimerDelay, NULL);
         p_kulka = new Kulka(hwnd, 0, 0, 1, 1, RGB(255,0,0), 30);
         
      return 0;
      case WM_TIMER:
         {
            RECT rc;
            static RECT stary_rc = {-1, -1, -1, -1};
            GetWindowRect(hwnd, &rc);
            RECT v;
            if (stary_rc.left < 0)
               SetRectEmpty(&v);
            else
            {
               v.left   = rc.left   - stary_rc.left;
               v.right  = rc.right  - stary_rc.right;
               v.bottom = rc.bottom - stary_rc.bottom;
               v.top    = rc.top    - stary_rc.top;
            }
            CopyRect(&stary_rc, &rc);

            GetClientRect(hwnd, &rc);
            HDC hdc;
            hdc = GetDC(hwnd);
               p_kulka->Usun(hdc, COLOR_BTNFACE);
               GdiFlush();
               p_kulka->Przesun(dt);
               p_kulka->SprawdzZderzenia(rc, v);
               p_kulka->Odrysuj(hdc);
            ReleaseDC(hwnd, hdc);
         }
         return 0;
      case WM_MOVING:
      case WM_SIZING:
      case WM_WINDOWPOSCHANGED:  // <-- to chyba mozna zrobic lepiej
         InvalidateRect(hwnd, 0, true);
         return true;

      case WM_PAINT:
         {
            PAINTSTRUCT ps;
            HDC hdc = BeginPaint(hwnd, &ps);

            p_kulka->Odrysuj(hdc);                         
            
            EndPaint(hwnd, &ps);
         }
         return 0;

      case WM_CLOSE:
         DestroyWindow(hwnd);
      break;
      case WM_DESTROY:
         KillTimer(hwnd, idTimer1);
         delete p_kulka;
         p_kulka = 0;
         PostQuitMessage(0);
      break;
      default:
         return DefWindowProc(hwnd, Message, wParam, lParam);
   }
   return 0;
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpCmdLine, int nCmdShow)
{
   WNDCLASSEX WndClass;
   HWND hwnd;
   MSG Msg;

   g_hInst = hInstance;

   WndClass.cbSize        = sizeof(WNDCLASSEX);
   WndClass.style         = 0, //CS_HREDRAW | CS_VREDRAW;
   WndClass.lpfnWndProc   = WndProc;
   WndClass.cbClsExtra    = 0;
   WndClass.cbWndExtra    = 0;
   WndClass.hInstance     = g_hInst;
   WndClass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
   WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
   WndClass.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
   WndClass.lpszMenuName  = NULL;
   WndClass.lpszClassName = g_szClassName;
   WndClass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

   if(!RegisterClassEx(&WndClass))
   {
      MessageBox(0, "Window Registration Failed!", "Error!",
         MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
      return 0;
   }

   hwnd = CreateWindowEx(
      WS_EX_CLIENTEDGE,
      g_szClassName,
      "Taka tam kulka...",
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT, 320, 240,
      NULL, NULL, g_hInst, NULL);

   if(hwnd == NULL)
   {
      MessageBox(0, "Window Creation Failed!", "Error!",
         MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
      return 0;
   }

   ShowWindow(hwnd, nCmdShow);
   UpdateWindow(hwnd);

   while(GetMessage(&Msg, NULL, 0, 0))
   {
      TranslateMessage(&Msg);
      DispatchMessage(&Msg);
   }
   return Msg.wParam;
}

