/* 
   Name: WinAnim
   Author: Brook Miles
   Description: Making an animation in windows
*/

#include <windows.h>

static char g_szClassName[] = "MyWindowClass";
static HINSTANCE g_hInst = NULL;

const UINT idTimer1 = 1;
const UINT idTimerSec = 2;
UINT nTimerDelay = 0;

HBITMAP hbmBall, hbmMask;
BITMAP bm;

int ballX, ballY;
int deltaX, deltaY;

int deltaValue = 4;



void DrawBall(HDC hdc, int lewy, int gorny)
{
   
   HDC hdcMemory = CreateCompatibleDC(hdc);

   HBITMAP hbm = (HBITMAP) SelectObject(hdcMemory, hbmMask);
   BitBlt(hdc, ballX - lewy, ballY - gorny, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCAND);

   SelectObject(hdcMemory, hbmBall);
   BitBlt(hdc, ballX - lewy, ballY - gorny, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCPAINT);

   SelectObject(hdcMemory, hbm);
   DeleteDC(hdcMemory);
}

void UpdateBall(HWND hwnd)
{
   RECT rc;
   GetClientRect(hwnd, &rc);

   ballX += deltaX;
   ballY += deltaY;

   if(ballX < 0){
      ballX = 0;
      deltaX = deltaValue;
   }
   else if(ballX + bm.bmWidth > rc.right){
      ballX = rc.right - bm.bmWidth;
      deltaX = -deltaValue;
   }
   if(ballY < 0){
      ballY = 0;
      deltaY = deltaValue;
   }
   else if(ballY + bm.bmHeight > rc.bottom){
      ballY = rc.bottom - bm.bmHeight;
      deltaY = -deltaValue;
   }
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{

   switch(Message)
   {      
      case WM_CREATE:
         hbmBall = LoadBitmap(g_hInst, "BALLBMP");
         hbmMask = LoadBitmap(g_hInst, "MASKBMP");
         if(!hbmBall || !hbmMask){
            MessageBox(hwnd, "Load of resources failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
         }

         GetObject(hbmBall, sizeof(bm), &bm);
         SetTimer(hwnd, idTimer1, nTimerDelay, NULL);
         SetTimer(hwnd, idTimerSec, 1000, NULL);

         ballX = 0;
         ballY = 0;
         deltaX = deltaValue;
         deltaY = deltaValue;
      break;
      

      case WM_TIMER:
      {
         static int licznik = 0;
         if(!hbmBall || !hbmMask)
           break;
         
         if (wParam == idTimerSec)
         {
            char bufor[128];
            wsprintf(bufor, "%d fps", licznik);
            SetWindowText(hwnd, bufor);      
            licznik = 0;
            break;
         }      
         licznik++;
         static RECT old_ball_rect = {ballX,ballY,-10,-10};
         if (old_ball_rect.right < 0)
         {
               old_ball_rect.right = ballX + bm.bmWidth;
               old_ball_rect.bottom = ballY + bm.bmHeight;
         }
           
         UpdateBall(hwnd);

         RECT ball_rect = {ballX, ballY, ballX + bm.bmWidth, ballY + bm.bmHeight};
         RECT suma;
         UnionRect(&suma, &old_ball_rect, &ball_rect);
            GetClientRect(hwnd, &suma);
         CopyRect(&old_ball_rect, &ball_rect);
         int szer = suma.right - suma.left+1;
         int wys = suma.bottom - suma.top+1;
         
         HDC hdc = GetDC(hwnd);
         HDC hdc_mem = CreateCompatibleDC (hdc);

         HBITMAP hbitmap = CreateCompatibleBitmap(hdc, szer, wys);
         HBITMAP hbm_old = (HBITMAP) SelectObject(hdc_mem, hbitmap);
         

         RECT rc = {0, 0, szer, wys};
         FillRect(hdc_mem, &rc, (HBRUSH)(COLOR_BTNFACE+1));
         DrawBall(hdc_mem, suma.left, suma.top);
            
         BitBlt(hdc, suma.left, suma.top, szer, wys, hdc_mem, 0, 0, SRCCOPY);
            
         DeleteObject(SelectObject(hdc_mem, hbm_old));
         DeleteDC (hdc_mem);         
                    
         ReleaseDC(hwnd, hdc);
         break;
      }
      case WM_PAINT:
         if(hbmBall && hbmMask)
         {
            PAINTSTRUCT ps;
            HDC hdcWindow;
            hdcWindow = BeginPaint(hwnd, &ps);

            DrawBall(hdcWindow, 0, 0);
            
            EndPaint(hwnd, &ps);
         }
      break;
      case WM_CLOSE:
         DestroyWindow(hwnd);
      break;
      case WM_DESTROY:
         KillTimer(hwnd, idTimer1);
         KillTimer(hwnd, idTimerSec);
         
         DeleteObject(hbmBall);
         DeleteObject(hbmMask);
         PostQuitMessage(0);
      break;
      default:
         return DefWindowProc(hwnd, Message, wParam, lParam);
   }
   return 0;
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpCmdLine, int nCmdShow)
{
   WNDCLASSEX WndClass;
   HWND hwnd;
   MSG Msg;

   g_hInst = hInstance;

   WndClass.cbSize        = sizeof(WNDCLASSEX);
   WndClass.style         =  0;
   WndClass.lpfnWndProc   = WndProc;
   WndClass.cbClsExtra    = 0;
   WndClass.cbWndExtra    = 0;
   WndClass.hInstance     = g_hInst;
   WndClass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
   WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
   WndClass.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
   WndClass.lpszMenuName  = NULL;
   WndClass.lpszClassName = g_szClassName;
   WndClass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

   if(!RegisterClassEx(&WndClass))
   {
      MessageBox(0, "Window Registration Failed!", "Error!",
         MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
      return 0;
   }

   hwnd = CreateWindowEx(
      WS_EX_CLIENTEDGE,
      g_szClassName,
      "A Bitmap Program",
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT, 320, 240,
      NULL, NULL, g_hInst, NULL);

   if(hwnd == NULL)
   {
      MessageBox(0, "Window Creation Failed!", "Error!",
         MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
      return 0;
   }

   ShowWindow(hwnd, nCmdShow);
   UpdateWindow(hwnd);

   while(GetMessage(&Msg, NULL, 0, 0))
   {
      TranslateMessage(&Msg);
      DispatchMessage(&Msg);
   }
   return Msg.wParam;
}

