#ifndef BITMAP_H__
#define BITMAP_H__

#if !defined(_WINDOWS_H) && !defined(_WINDOWS_)
#include <windows.h>
#endif

class BitMap
{
public:
   BitMap(HINSTANCE hinst, const char* nazwa);
   BitMap(HINSTANCE hinst, int nr);
   BitMap(HINSTANCE hinst, const char* nazwa, const char* maska);
   BitMap(HINSTANCE hinst, int nr, int nr_maski);
    ~BitMap();
    
   int szer() const;
   int wys()  const;
   
   void rysuj(HDC hdc, int x0, int y0, bool centralnie = true) const;
   void rysuj(HDC hdc, POINT point, bool centralnie = true) const;

 
private:
   HBITMAP _bitmap;
   HBITMAP _maska;
   SIZE _rozmiar;

   void _ustaw_rozmiar();
};


// DEFINICJE FUNKCJI 

inline BitMap::BitMap(HINSTANCE hinst, const char* nazwa)
: _maska(0) 
{  
   _bitmap = ::LoadBitmap(hinst, nazwa);
   _ustaw_rozmiar();
}

inline BitMap::BitMap(HINSTANCE hinst, int nr)
:  _maska(0)
{ 
   _bitmap = ::LoadBitmap( hinst, MAKEINTRESOURCE(nr) ); 
   _ustaw_rozmiar();
}

inline BitMap::BitMap(HINSTANCE hinst, const char* nazwa, const char* maska)
{ 
   _bitmap = ::LoadBitmap(hinst, nazwa); 
   _maska  = ::LoadBitmap(hinst, maska);
   _ustaw_rozmiar();
}

inline BitMap::BitMap(HINSTANCE hinst, int nr, int nr_maski)   
{  
   _bitmap = ::LoadBitmap(hinst, MAKEINTRESOURCE(nr));
   _maska  = ::LoadBitmap(hinst, MAKEINTRESOURCE(nr_maski));
   _ustaw_rozmiar();
}

inline BitMap::~BitMap()
{
   if (_bitmap)
      ::DeleteObject(_bitmap);
   if (_maska)
      ::DeleteObject(_maska);   
}

inline int BitMap::szer() const { return _rozmiar.cx; }

inline int BitMap::wys()  const { return _rozmiar.cy; }

inline void BitMap::rysuj(HDC hdc, int x0, int y0, bool centralnie) const
{
   if (centralnie)
   {
      x0 -= szer()/2;
      y0 -= wys()/2;
   }
   HDC hdc_mem = ::CreateCompatibleDC(hdc);
   if (_maska == 0)
   {
      HBITMAP hb0 = (HBITMAP)SelectObject(hdc_mem, _bitmap);
      ::BitBlt(hdc, x0, y0, szer(), wys(), hdc_mem, 0, 0, SRCCOPY);
      SelectObject(hdc_mem, hb0);
   }
   else
   {
      HBITMAP hb0 = (HBITMAP)SelectObject(hdc_mem, _maska);
      ::BitBlt(hdc, x0, y0, szer(), wys(), hdc_mem, 0, 0, SRCAND);
      SelectObject(hdc_mem, _bitmap);
      ::BitBlt(hdc, x0, y0, szer(), wys(), hdc_mem, 0, 0, SRCPAINT);
      SelectObject(hdc_mem, hb0);            
   }
   
   DeleteDC(hdc_mem);
}

inline void BitMap::rysuj(HDC hdc, POINT point, bool centralnie)  const
{
   rysuj(hdc, point.x, point.y, centralnie);
}


inline void BitMap::_ustaw_rozmiar()
{
   BITMAP bm;
   ::GetObject(_bitmap, sizeof(bm), &bm);
   _rozmiar.cx = bm.bmWidth;
   _rozmiar.cy = bm.bmHeight;
}
#endif

