#include <vector>
#include <windows.h>
#include "zasoby.h"

#include "BitMap.h"
#include "Cudo.h"


/*  Declare Windows procedure  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);

/*  Make the class name into a global variable  */
char szClassName[ ] = "WindowsApp";

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)

{
    HWND hwnd;               /* This is the handle for our window */
    MSG messages;            /* Here messages to the application are saved */
    WNDCLASSEX wincl;        /* Data structure for the windowclass */

    /* The Window structure */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc = WindowProcedure;      /* This function is called by windows */
    wincl.style = 0;//CS_HREDRAW | CS_VREDRAW;       
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL;                 /* No menu */
    wincl.cbClsExtra = 0;                      /* No extra bytes after the window class */
    wincl.cbWndExtra = 0;                      /* structure or the window instance */
    /* Use Windows's default color as the background of the window */
    wincl.hbrBackground = 0;//(HBRUSH) COLOR_BACKGROUND;

    /* Register the window class, and if it fails quit the program */
    if (!RegisterClassEx (&wincl))
        return 0;

    /* The class is registered, let's create the program*/
    hwnd = CreateWindowEx (
           0,                   /* Extended possibilites for variation */
           szClassName,         /* Classname */
           "Windows App",       /* Title Text */
           WS_OVERLAPPEDWINDOW, /* default window */
           CW_USEDEFAULT,       /* Windows decides the position */
           CW_USEDEFAULT,       /* where the window ends up on the screen */
           544,                 /* The programs width */
           375,                 /* and height in pixels */
           HWND_DESKTOP,        /* The window is a child-window to desktop */
           NULL,                /* No menu */
           hThisInstance,       /* Program Instance handler */
           NULL                 /* No Window Creation data */
           );

    /* Make the window visible on the screen */
    ShowWindow (hwnd, nFunsterStil);

    /* Run the message loop. It will run until GetMessage() returns 0 */
    while (GetMessage (&messages, NULL, 0, 0))
    {
        /* Translate virtual-key messages into character messages */
        TranslateMessage(&messages);
        /* Send message to WindowProcedure */
        DispatchMessage(&messages);
    }

    /* The program return-value is 0 - The value that PostQuitMessage() gave */
    return messages.wParam;
}

void on_paint(HDC hdc, HWND hwnd, std::vector<Cudo*> tab_cudow)
{
    HINSTANCE hinst = (HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE);
    BitMap kafelek(hinst, TLO);
    RECT rect;
    ::GetClientRect(hwnd, &rect);
    for (int x = 0; x <= rect.right/kafelek.szer(); x++)
      for (int y = 0; y <= rect.bottom/kafelek.wys(); y++)
        kafelek.rysuj(hdc, x*kafelek.szer(), y*kafelek.wys(), false);
    
    for (unsigned i = 0; i < tab_cudow.size(); i++)
       tab_cudow[i]->rysuj(hdc);
    
}



/*  This function is called by the Windows function DispatchMessage()  */

LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static std::vector<Cudo*> tab_cudow;

    static BitMap* pBitMap_piramida = 0;
    static BitMap* pBitMap_okragla = 0;


    switch (message)                  /* handle the messages */
    {
        case WM_CREATE:
        {
           LPCREATESTRUCT p = (LPCREATESTRUCT) lParam;
           HINSTANCE  hinst = p->hInstance;
           pBitMap_piramida = new BitMap(hinst, TROJKAT);
           pBitMap_okragla  = new BitMap(hinst, KULKA, MASKA_KULKI);

           break;
        }
        case WM_DESTROY:
        {
            for (unsigned i = 0; i < tab_cudow.size(); i++)
            {
               delete tab_cudow[i];
            }
            delete pBitMap_piramida;
            delete pBitMap_okragla;
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            break;
        }
        case WM_PAINT:
        {
           PAINTSTRUCT ps;
           HDC hdc = ::BeginPaint(hwnd, &ps);
           on_paint(hdc, hwnd, tab_cudow);
           
           ::EndPaint(hwnd, &ps);  
           break;  
        }

        case WM_LBUTTONDOWN: 
        case WM_RBUTTONDOWN:
        {
           BitMap* p = (message == WM_LBUTTONDOWN ) ? pBitMap_piramida : pBitMap_okragla; 
           POINT point = {LOWORD(lParam), HIWORD(lParam)};
           tab_cudow.push_back(new Cudo(hwnd, point, p));
           HDC hdc = GetDC(hwnd);
           tab_cudow.back()->rysuj(hdc);
           ReleaseDC(hwnd, hdc);
           break;
        }
        case WM_CHAR:
        case WM_SYSKEYDOWN: // "undo"
        {
            bool ctrl_z = GetKeyState(VK_CONTROL) < 0  
                          && message == WM_CHAR    
                          && (TCHAR) wParam == 26;
            bool alt_bkspace = message == WM_SYSKEYDOWN 
                               && (TCHAR) wParam == VK_BACK;
            if (  ctrl_z || alt_bkspace )
            {
               if (!tab_cudow.empty())
               {
                  Cudo* p = tab_cudow.back();
                  RECT rect = p->rect();
                  tab_cudow.pop_back();
                  delete p;
                  InvalidateRect(hwnd, &rect, false);
               }
               
            }
            if (message == WM_SYSKEYDOWN)
               return DefWindowProc (hwnd, message, wParam, lParam);
            break;
        }
        
        default:                      /* for messages that we don't deal with */
            return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return 0;
}

