#include <cmath>
#include <cstdio>
#include <windows.h>
#include "resource.h"
#include "dialog_trojmian.h"
#include "struktura.h"

template <typename T> 
inline void
zamien(T & x, T & y)
{
    T tmp (x);
    x = y;
    y = tmp;   
}

BOOL CALLBACK 
DialogTrojmianProcedure (HWND hdlg, UINT message,
                         WPARAM wParam, LPARAM lParam)
{
    const int max_rozmiar = 11;
    const int max_dokladnosc = 15;
       
    static bool gotowe[3];
    static int parametr[3] = {0};
    static unsigned int dokladnosc;


    switch(message)
    {
       case WM_INITDIALOG:
       {
          gotowe[0] = gotowe[1] = gotowe[2] = false;
          dokladnosc = 6;
          
          SendMessage(GetDlgItem(hdlg, IDC_EDIT_A), EM_SETLIMITTEXT, max_rozmiar, 0);
          SendMessage(GetDlgItem(hdlg, IDC_EDIT_B), EM_SETLIMITTEXT, max_rozmiar, 0);
          SendMessage(GetDlgItem(hdlg, IDC_EDIT_C), EM_SETLIMITTEXT, max_rozmiar, 0);
          SendMessage(GetDlgItem(hdlg, IDC_EDIT_DOKLADNOSC), EM_SETLIMITTEXT, 1, 0);
          SetDlgItemInt(hdlg, IDC_EDIT_DOKLADNOSC, dokladnosc, false);
          SetFocus(GetDlgItem(hdlg, IDC_EDIT_A));
          
          return true; // automatyczny fokus
       }

       case WM_HSCROLL:
       {
          int nScrollCode = (int) LOWORD(wParam);
          switch (nScrollCode)
          {
             case SB_LINELEFT: 
             case SB_LINERIGHT: 
             {
                bool zmienione = false;
                if (nScrollCode == SB_LINELEFT && dokladnosc > 1) 
                {
                   dokladnosc--; 
                   zmienione = true;
                }
                if (nScrollCode == SB_LINERIGHT && dokladnosc < max_dokladnosc) 
                {
                   dokladnosc++; 
                   zmienione = true;
                }
                if (!zmienione) 
                   break;
                SetDlgItemInt(hdlg, IDC_EDIT_DOKLADNOSC, dokladnosc, false);
//                if (gotowe[0] && gotowe[1] && gotowe[2])
//                {
//                   EnableWindow(GetDlgItem(hdlg, IDOK), true);
//                   SetDlgItemText(hdlg, IDCANCEL, "Anuluj");
//                   SetDlgItemText(hdlg, IDC_EDIT_X0, "");   
//                   SetDlgItemText(hdlg, IDC_EDIT_X1, "");
//                }  
                if (gotowe[0] && gotowe[1] && gotowe[2])           
                    SendMessage(hdlg, WM_COMMAND, IDOK, 0);
                break;
             } //case
          }//switch
          return true;
       }

     
       case WM_CTLCOLORSTATIC :
       {
          int i = GetWindowLong ((HWND) lParam, GWL_ID) ;
          
          if (i == IDC_EDIT_X0 || i == IDC_EDIT_X1) 
          {
             SetTextColor ((HDC) wParam, RGB(255,0,0)) ;
//             SetBkColor((HDC) wParam, RGB(0,0,255));
             return (int)GetStockObject(WHITE_BRUSH);
          }
          break ;
       }
          

       case WM_COMMAND:
          int przyczyna = HIWORD(wParam); // notification code 
          int kto = LOWORD(wParam);      // item, control, or accelerator identifier 
          HWND hwndCtrl = (HWND) lParam; // handle of control 

          switch(kto)
          {
          case IDCANCEL:
             EndDialog (hdlg, 0);
             break;
          case IDC_EDIT_A:
          case IDC_EDIT_B:
          case IDC_EDIT_C:
             {
                int indeks;
                if (kto == IDC_EDIT_A)
                   indeks = 0;
                else if (kto == IDC_EDIT_B)
                   indeks = 1;
                else indeks = 2;
                   
                switch(przyczyna)
                {
                case EN_CHANGE:
                   {
                      BOOL ok;
                      parametr[indeks] = GetDlgItemInt(hdlg, kto, &ok, TRUE);
                      gotowe[indeks] = ok;
                      bool mozna_liczyc = gotowe[0] && gotowe[1] && gotowe[2];
                      EnableWindow(GetDlgItem(hdlg, IDOK), mozna_liczyc);
                      SetDlgItemText(hdlg, IDCANCEL, "Anuluj");
                      SetDlgItemText(hdlg, IDC_EDIT_X0, "");
                      SetDlgItemText(hdlg, IDC_EDIT_X1, "");
                   }
                } 

                break;
             }

          case IDOK:
             {
               double a = parametr[0];
               double b = parametr[1];
               double c = parametr[2];
               QuadraticData qd = {0};
               qd.a = parametr[0];
               qd.b = parametr[1];
               qd.c = parametr[2];

               double delta = b*b - 4.0*a*c;
               char bufor[2][500] = {"", ""};
                                 
               if (a != 0)
               {
                  qd.valid_roots = 2;
                  bool zespolone = delta < 0;
                  qd.complex_roots = zespolone;
                  if (zespolone)
                     delta = -delta;
                  delta = sqrt(delta);
                  
                  if (!zespolone)
                  {
                     double x0 = (-b - delta)/(2.0*a);
                     double x1 = (-b + delta)/(2.0*a);
                     sprintf(bufor[0], "%#.*g", dokladnosc, x0);
                     sprintf(bufor[1], "%#.*g", dokladnosc, x1);
                     qd.x0 = x0;
                     qd.x1 = x1;
                  }
                  else
                  {
                     double re = -0.5*b/a;
                     double im = 0.5*delta/a;
                       
                     sprintf(bufor[0], "%#.*g + %#.*gi", 
                              dokladnosc, re, dokladnosc, im);
                     sprintf(bufor[1], "%#.*g - %#.*gi", 
                              dokladnosc, re, dokladnosc, im);
                  }
                  
               }
               else if (b != 0)
               {
                  qd.valid_roots = 1;
                  double x0= -c/b;
                  sprintf(bufor[0], "%.*g", dokladnosc, x0);
                  sprintf(bufor[1], "nie istnieje" );

               }
               else if (c != 0)
               {  
                  qd.valid_roots = 0;
                  sprintf(bufor[0], "Rwnanie nie ma rozwiza!");
               }
               else
               {
                  qd.valid_roots = 3;               
                  sprintf(bufor[0], "Kada liczba spenia to rwnanie");
               }


               SetDlgItemText(hdlg, IDC_EDIT_X0, bufor[0]);   
               SetDlgItemText(hdlg, IDC_EDIT_X1, bufor[1]);
               EnableWindow(GetDlgItem(hdlg, IDOK), false);
               SetDlgItemText(hdlg, IDCANCEL, "Zakocz");
               SetFocus(GetDlgItem(hdlg, IDCANCEL));
               
               
               SendMessage(GetParent(hdlg), UM_SET_DATA, (WPARAM)&qd, 0); 

 
               break;
             }

          default:
             return false;
          }
          return true;
    }
    return false;
}
                         


